"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
exports.__esModule = true;
exports.BlockController = void 0;
var utils_1 = require("./utils");
var fs = require('fs');
var Block = /** @class */ (function () {
    function Block(t, n, c) {
        this.type = t;
        this.name = n;
        this.content = c;
    }
    return Block;
}());
var ExtendInfo = /** @class */ (function () {
    function ExtendInfo() {
        this.isExtending = false;
        this.extendedFile = '';
        this.fileModifiedContent = '';
        this.errors = [];
    }
    return ExtendInfo;
}());
var BlockController = /** @class */ (function () {
    function BlockController() {
    }
    BlockController.getExtendingInfo = function (fileContent) {
        var extendingInfo = new ExtendInfo();
        if (!utils_1.Patterns.extendRegex.test(fileContent)) {
            extendingInfo.isExtending = false;
            extendingInfo.fileModifiedContent = fileContent;
            return extendingInfo;
        }
        var extendDirectives = fileContent.match(utils_1.Patterns.extendRegex);
        if (extendDirectives.length > 1) {
            extendingInfo.errors.push("hi");
            return extendingInfo;
        }
        extendingInfo.isExtending = true;
        extendingInfo.fileModifiedContent = fileContent.replace(utils_1.Patterns.extendRegex, "");
        extendingInfo.extendedFile = extendDirectives[0].replace(/@extends +/g, "").replace(/('|"| )/g, "");
        return extendingInfo;
    };
    BlockController.ConvertToBlocks = function (fileContent) {
        var info = this.getExtendingInfo(fileContent);
        fileContent = info.fileModifiedContent;
        var blockHeaders = fileContent.match(utils_1.Patterns.blockHeaderRegex);
        console.log(blockHeaders);
        var fileAsBlocks = fileContent.split(utils_1.Patterns.blockHeaderRegex)
            .filter(function (x) { return x; })
            .map(function (blockContent, i) {
            if (i === 0)
                return [new Block("none", "none", blockContent)];
            i--;
            console.log(blockContent, i);
            var blockType = 'regular';
            blockType = blockHeaders[i].includes("prepend") ? "prepend" : blockType;
            blockType = blockHeaders[i].includes("append") ? "append" : blockType;
            var blocks = blockContent.split(utils_1.Patterns.blockFooterRegex);
            if (blocks.length !== 2) {
                //throw somekind of error;
            }
            return [
                new Block(blockType, blockHeaders[i]
                    .replace(utils_1.Patterns.blockHeaderNameExtractRegex, "")
                    .replace(/ */g, ""), blocks[0]),
                new Block("none", "none", blocks[1])
            ];
        })
            .reduce(function (prev, current) {
            if (current instanceof Array)
                return __spreadArray(__spreadArray([], prev), current);
            return __spreadArray(__spreadArray([], prev), [current]);
        }, []);
        var blocksLevels = [fileAsBlocks];
        if (info.isExtending) {
            var extendedFileContent = fs.readFileSync(info.extendedFile, { encoding: 'utf8', flag: 'r' });
            blocksLevels = __spreadArray(__spreadArray([], blocksLevels), this.ConvertToBlocks(extendedFileContent));
        }
        return blocksLevels;
    };
    BlockController.mergeBlocks = function (blocksLevels) {
        var superBlock = blocksLevels.pop();
        var newFileContent = superBlock.reduce(function (prev, current) {
            var findLastBlockByNameAndType = function (name, type, fallback) {
                var lastAccurOfBlock;
                blocksLevels.reverse();
                blocksLevels.forEach(function (level) {
                    level.forEach(function (block) {
                        if (block.name === name
                            && block.type === type
                            && type !== "none") {
                            lastAccurOfBlock = block;
                        }
                    });
                });
                return lastAccurOfBlock || fallback || '';
            };
            var fileContentUpTillNow = '';
            fileContentUpTillNow += (prev || '');
            if (current.type === "none") {
                return fileContentUpTillNow + current.content;
            }
            else {
                fileContentUpTillNow += findLastBlockByNameAndType(current.name, "prepend").content || '';
                fileContentUpTillNow += (findLastBlockByNameAndType(current.name, current.type) || current).content;
                fileContentUpTillNow += findLastBlockByNameAndType(current.name, "append").content || '';
            }
            return fileContentUpTillNow;
        }, "");
        return newFileContent;
    };
    return BlockController;
}());
exports.BlockController = BlockController;
